// *****************************
// ** zx81modestate.h         **
// ** Class Definitions       **
// ** For Mode Switch  & LEDs **
// *****************************

// Defines LED / Mode switch panel and its behaviour.
// Three LEDs are configured to report on keyboard mode and states.
// 
// In STANDARD mode:
//		Left LED on = GRAPHICS state.
//		Middle LED on = FUNCTION state.
//		Right LED on = NORMAL state.
//
// In EMULATOR mode:
//		Left and Right LEDs = on.

class ModeState {
private:
	KEYMODES eKeyboardMode = STANDARD;
	KEYSTATES eKeyboardState = NORMAL;
	byte bModeSwitchPin;
	byte bGraphicsPin;
	byte bFunctionPin;
	byte bModePin;
	
	bool boDebounce = false;
	
	void SetMode(){
		if (digitalRead(bModeSwitchPin) == HIGH && boDebounce == false) {
			//Serial.println("high ");
			if (eKeyboardMode == STANDARD){
				eKeyboardMode = EMULATOR;
			} else {
				eKeyboardMode = STANDARD;
			}
			
			delay(DEBOUNCE_DELAY_REPEAT);
			boDebounce = true;
		} else if (digitalRead(bModeSwitchPin) == LOW) {
			boDebounce = false;
		}
	}
	
public:
	ModeState (byte, byte, byte, byte);
	
	void SetState(KEYSTATES eKeystate){
		
		eKeyboardState = eKeystate;
		
		// Check and set mode if Mode switch is pressed
		SetMode();
		
		if (eKeyboardMode != EMULATOR){
			switch (eKeyboardState) {
				
				case FUNCTION:
					digitalWrite(bGraphicsPin, LOW);
					digitalWrite(bFunctionPin, HIGH);
					digitalWrite(bModePin, LOW);
					break;
					
				case GRAPHICS:
					digitalWrite(bGraphicsPin, HIGH);
					digitalWrite(bFunctionPin, LOW);
					digitalWrite(bModePin, LOW);
					break;
					
				default:
					digitalWrite(bGraphicsPin, LOW);
					digitalWrite(bFunctionPin, LOW);
					digitalWrite(bModePin, HIGH);
					eKeyboardState = NORMAL;
					break;
			}
		} else {
			digitalWrite(bGraphicsPin, HIGH);
			digitalWrite(bFunctionPin, LOW);
			digitalWrite(bModePin, HIGH);
			eKeyboardState = NORMAL;
		}
	}
	
	KEYMODES GetMode(){
		return eKeyboardMode;
	}
	
	KEYSTATES GetState(){
		return eKeyboardState;
	}
	
	void SetMode(KEYMODES eMode){
                eKeyboardMode = eMode;
	}
	
};

// ModeState constructor.
ModeState::ModeState (byte ModeSwitchPin, byte GraphicsPin, byte FunctionPin, byte ModePin){
	
	bModeSwitchPin = ModeSwitchPin;
	bGraphicsPin = GraphicsPin;
	bFunctionPin = FunctionPin;
	bModePin = ModePin;
	
	pinMode(bModeSwitchPin, INPUT);
	pinMode(bGraphicsPin, OUTPUT);
	pinMode(bFunctionPin, OUTPUT);
	pinMode(bModePin, OUTPUT);
	
}
